@@ -4,14 +4,13 @@
 #include "qplatformmediaplayer_p.h"
 #include <private/qmediaplayer_p.h>
 #include "qmediaplayer.h"
-#include "qplatformmediadevices_p.h"
+#include "qplatformaudiodevices_p.h"
 #include "qplatformmediaintegration_p.h"
 
 QT_BEGIN_NAMESPACE
 
 QPlatformMediaPlayer::QPlatformMediaPlayer(QMediaPlayer *parent) : player(parent)
 {
-    QPlatformMediaIntegration::instance()->mediaDevices()->prepareAudio();
 }
 
 QPlatformMediaPlayer::~QPlatformMediaPlayer() = default;
@@ -32,9 +31,36 @@ void QPlatformMediaPlayer::mediaStatusChanged(QMediaPlayer::MediaStatus status)
     player->d_func()->setStatus(status);
 }
 
-void QPlatformMediaPlayer::error(int error, const QString &errorString)
+void QPlatformMediaPlayer::error(QMediaPlayer::Error error, const QString &errorString)
 {
-    player->d_func()->setError(QMediaPlayer::Error(error), errorString);
+    player->d_func()->setError(error, errorString);
+}
+
+QPlatformMediaPlayer::PitchCompensationAvailability
+QPlatformMediaPlayer::pitchCompensationAvailability() const
+{
+    return PitchCompensationAvailability::Unavailable;
+}
+
+void QPlatformMediaPlayer::setPitchCompensation(bool /*enabled*/)
+{
+    qWarning() << "QMediaPlayer::setPitchCompensation not supported on this QtMultimedia "
+                  "backend";
+}
+
+bool QPlatformMediaPlayer::pitchCompensation() const
+{
+    return false;
+}
+
+void QPlatformMediaPlayer::pitchCompensationChanged(bool enabled) const
+{
+    emit player->pitchCompensationChanged(enabled);
+}
+
+QPlaybackOptions QPlatformMediaPlayer::playbackOptions() const
+{
+    return player->playbackOptions();
 }
 
 QT_END_NAMESPACE
