@@ -132,16 +132,6 @@ void QMediaPlayerPrivate::setMedia(const QUrl &media, QIODevice *stream)
     if (!control)
         return;
 
-    auto setErrorFn = [&](
-        QMediaPlayer::MediaStatus status,
-        QMediaPlayer::Error err,
-        const QString &errString)
-    {
-        control->setMedia(QUrl(), nullptr);
-        control->mediaStatusChanged(status);
-        control->error(err, errString);
-    };
-
     std::unique_ptr<QFile> file;
 
     // Back ends can't play qrc files directly.
@@ -154,10 +144,9 @@ void QMediaPlayerPrivate::setMedia(const QUrl &media, QIODevice *stream)
         file.reset(new QFile(QLatin1Char(':') + media.path()));
         if (!file->open(QFile::ReadOnly)) {
             file.reset();
-            setErrorFn(
-                QMediaPlayer::InvalidMedia,
-                QMediaPlayer::ResourceError,
-                QMediaPlayer::tr("Attempting to play invalid Qt resource"));
+            control->setMedia(QUrl(), nullptr);
+            control->mediaStatusChanged(QMediaPlayer::InvalidMedia);
+            control->error(QMediaPlayer::ResourceError, QMediaPlayer::tr("Attempting to play invalid Qt resource"));
 
         } else if (control->streamPlaybackSupported()) {
             control->setMedia(media, file.get());
@@ -166,23 +155,17 @@ void QMediaPlayerPrivate::setMedia(const QUrl &media, QIODevice *stream)
 #if defined(Q_OS_ANDROID)
             QString tempFileName = QDir::tempPath() + media.path();
             QDir().mkpath(QFileInfo(tempFileName).path());
-            std::unique_ptr<QTemporaryFile> tempFile { QTemporaryFile::createNativeFile(*file) };
-            if (tempFile.get() == nullptr) {
-                setErrorFn(
-                    QMediaPlayer::InvalidMedia,
-                    QMediaPlayer::ResourceError,
-                    QMediaPlayer::tr("Failed to establish temporary file during playback"));
-                return;
-            }
-            if (!tempFile->rename(tempFileName)) {
-                setErrorFn(
-                    QMediaPlayer::InvalidMedia,
-                    QMediaPlayer::ResourceError,
-                    QStringLiteral("Could not rename temporary file to: %1").arg(tempFileName));
+            QTemporaryFile *tempFile = QTemporaryFile::createNativeFile(*file);
+            if (tempFile == nullptr) {
+                control->setMedia(QUrl(), nullptr);
+                control->mediaStatusChanged(QMediaPlayer::InvalidMedia);
+                control->error(QMediaPlayer::ResourceError, QMediaPlayer::tr("Failed to establish temporary file during playback"));
                 return;
             }
+            if (!tempFile->rename(tempFileName))
+                qWarning() << "Could not rename temporary file to:" << tempFileName;
 #else
-            std::unique_ptr<QTemporaryFile> tempFile = std::make_unique<QTemporaryFile>();
+            QTemporaryFile *tempFile = new QTemporaryFile;
 
             // Preserve original file extension, some back ends might not load the file if it doesn't
             // have an extension.
@@ -192,10 +175,10 @@ void QMediaPlayerPrivate::setMedia(const QUrl &media, QIODevice *stream)
 
             // Copy the qrc data into the temporary file
             if (!tempFile->open()) {
-                setErrorFn(
-                    QMediaPlayer::InvalidMedia,
-                    QMediaPlayer::ResourceError,
-                    tempFile->errorString());
+                control->setMedia(QUrl(), nullptr);
+                control->mediaStatusChanged(QMediaPlayer::InvalidMedia);
+                control->error(QMediaPlayer::ResourceError, tempFile->errorString());
+                delete tempFile;
                 qrcFile.reset();
                 return;
             }
@@ -208,7 +191,7 @@ void QMediaPlayerPrivate::setMedia(const QUrl &media, QIODevice *stream)
             }
             tempFile->close();
 #endif
-            file = std::move(tempFile);
+            file.reset(tempFile);
             control->setMedia(QUrl(QUrl::fromLocalFile(file->fileName())), nullptr);
 #else
             qWarning("Qt was built with -no-feature-temporaryfile: playback from resource file is not supported!");
@@ -1096,6 +1079,145 @@ QMediaMetaData QMediaPlayer::metaData() const
     return d->control ? d->control->metaData() : QMediaMetaData{};
 }
 
+/*!
+    \qmlproperty bool QtMultimedia::MediaPlayer::pitchCompensation
+    \since 6.10
+
+    This property holds whether pitch compensation is enabled.
+*/
+
+/*!
+    \property QMediaPlayer::pitchCompensation
+    \brief The pitch compensation status of the media player.
+    \since 6.10
+
+    Indicates whether pitch compensation is enabled. When enabled, changing the playback rate
+    will not affect the pitch of the audio signal.
+
+    \note The pitch compensation will increase the CPU load of the QMediaPlayer.
+
+    By default is \c{true} if pitch compensation, is available, else \c{false}.
+*/
+
+/*!
+    Returns the state of pitch compensation.
+    \since 6.10
+*/
+bool QMediaPlayer::pitchCompensation() const
+{
+    Q_D(const QMediaPlayer);
+    return d->control ? d->control->pitchCompensation() : false;
+}
+
+/*!
+    Sets the state (\a enabled or disabled) of pitch compensation. This only
+    has an effect if the audio pitch compensation can be configured on the
+    backend at runtime.
+    \since 6.10
+*/
+void QMediaPlayer::setPitchCompensation(bool enabled) const
+{
+    Q_D(const QMediaPlayer);
+    if (d->control)
+        d->control->setPitchCompensation(enabled);
+}
+
+/*!
+    \enum QMediaPlayer::PitchCompensationAvailability
+    \since 6.10
+
+    Availablility of pitch compensation.
+
+    Different backends have different behavior regarding pitch compensation when changing
+    playback rate.
+
+    \value AlwaysOn The media player is always performing pitch compensation.
+    \value Available The media player can be configured to use pitch compensation.
+        If pitch compensation is available on the current platform, it will be enabled by default,
+        but users can disable if needed.
+    \value Unavailable The media player is not able to perform pitch compensation
+        on the current platform.
+*/
+
+/*!
+    \qmlproperty enumeration QtMultimedia::MediaPlayer::pitchCompensationAvailability
+    \since 6.10
+
+    Indicates the availability of pitch compensation of the \c MediaPlayer on the current backend.
+    The enumeration \c PitchCompensationAvailability is scoped.
+
+    \qmlenumeratorsfrom QMediaPlayer::PitchCompensationAvailability
+*/
+
+/*!
+    \property QMediaPlayer::pitchCompensationAvailability
+    \brief The pitch compensation availability of the current QtMultimedia backend.
+    \since 6.10
+
+    Indicates the availability of pitch compensation of the QMediaPlayer on the current backend.
+
+    \note Different backends may have different behavior.
+
+    For more information, see \l{QMediaPlayer::PitchCompensationAvailability}.
+*/
+
+/*!
+    Returns availability of pitch compensation of the current backend.
+    \since 6.10
+*/
+
+QMediaPlayer::PitchCompensationAvailability QMediaPlayer::pitchCompensationAvailability() const
+{
+    Q_D(const QMediaPlayer);
+    return d->control->pitchCompensationAvailability();
+}
+
+/*!
+    \qmlproperty playbackOptions MediaPlayer::playbackOptions
+    \since 6.10
+
+    This property exposes the \l playbackOptions API that gives low-level control of media playback
+    options. Although we strongly recommend to rely on the default settings of \l MediaPlayer,
+    this API can be used to optimize media playback for specific use cases where the default
+    options are not ideal.
+
+    Playback options take effect the next time \l MediaPlayer::source is changed.
+*/
+
+/*!
+    \property QMediaPlayer::playbackOptions
+    \brief Advanced playback options used to configure media playback and decoding.
+    \since 6.10
+
+    This property exposes the \l QPlaybackOptions API that gives low-level control of media
+    playback options. Although we strongly recommend to rely on the default settings of
+    \l QMediaPlayer, this API can be used to optimize media playback for specific use cases where
+    the default options are not ideal.
+
+    Playback options take effect the next time \l QMediaPlayer::setSource() is called.
+*/
+
+QPlaybackOptions QMediaPlayer::playbackOptions() const
+{
+    Q_D(const QMediaPlayer);
+    return d->playbackOptions;
+}
+
+void QMediaPlayer::setPlaybackOptions(const QPlaybackOptions &options)
+{
+    Q_D(QMediaPlayer);
+    if (std::exchange(d->playbackOptions, options) != options)
+        emit playbackOptionsChanged();
+}
+
+void QMediaPlayer::resetPlaybackOptions()
+{
+    Q_D(QMediaPlayer);
+    QPlaybackOptions defaultOptions{ };
+    if (std::exchange(d->playbackOptions, defaultOptions) != defaultOptions)
+        emit playbackOptionsChanged();
+}
+
 // Enums
 /*!
     \enum QMediaPlayer::PlaybackState
