@@ -1,55 +1,56 @@
 // Copyright (C) 2016 The Qt Company Ltd.
 // SPDX-License-Identifier: LicenseRef-Qt-Commercial OR LGPL-3.0-only OR GPL-2.0-only OR GPL-3.0-only
 
-#include <qgstreamercamera_p.h>
+#include <mediacapture/qgstreamercamera_p.h>
 
-#include <qcameradevice.h>
-#include <qgst_debug_p.h>
+#include <QtMultimedia/qcameradevice.h>
+#include <QtMultimedia/qmediacapturesession.h>
+#include <QtMultimedia/private/qcameradevice_p.h>
+#include <QtCore/qdebug.h>
+
+#include <common/qgst_debug_p.h>
 #include <qgstreamervideodevices_p.h>
 #include <qgstreamerintegration_p.h>
-#include <qmediacapturesession.h>
 
 #if QT_CONFIG(linux_v4l)
 #include <linux/videodev2.h>
 #include <private/qcore_unix_p.h>
 #endif
 
-#include <QtCore/qdebug.h>
 
 QT_BEGIN_NAMESPACE
 
-QMaybe<QPlatformCamera *> QGstreamerCamera::create(QCamera *camera)
+q23::expected<QPlatformCamera *, QString> QGstreamerCamera::create(QCamera *camera)
 {
-    QGstElement videotestsrc = QGstElement::createFromFactory("videotestsrc");
-    if (!videotestsrc)
-        return errorMessageCannotFindElement("videotestsrc");
+    static const auto error = qGstErrorMessageIfElementsNotAvailable(
+            "videotestsrc", "capsfilter", "videoconvert", "videoscale", "identity");
+    if (error)
+        return q23::unexpected{ *error };
 
-    QGstElement capsFilter = QGstElement::createFromFactory("capsfilter", "videoCapsFilter");
-    if (!capsFilter)
-        return errorMessageCannotFindElement("capsfilter");
-
-    QGstElement videoconvert = QGstElement::createFromFactory("videoconvert", "videoConvert");
-    if (!videoconvert)
-        return errorMessageCannotFindElement("videoconvert");
-
-    QGstElement videoscale = QGstElement::createFromFactory("videoscale", "videoScale");
-    if (!videoscale)
-        return errorMessageCannotFindElement("videoscale");
-
-    return new QGstreamerCamera(videotestsrc, capsFilter, videoconvert, videoscale, camera);
+    return new QGstreamerCamera(camera);
 }
 
-QGstreamerCamera::QGstreamerCamera(QGstElement videotestsrc, QGstElement capsFilter,
-                                   QGstElement videoconvert, QGstElement videoscale,
-                                   QCamera *camera)
-    : QPlatformCamera(camera),
-      gstCamera(std::move(videotestsrc)),
-      gstCapsFilter(std::move(capsFilter)),
-      gstVideoConvert(std::move(videoconvert)),
-      gstVideoScale(std::move(videoscale))
+QGstreamerCamera::QGstreamerCamera(QCamera *camera)
+    : QGstreamerCameraBase(camera),
+      gstCameraBin{
+          QGstBin::create("camerabin"),
+      },
+      gstCamera{
+          QGstElement::createFromFactory("videotestsrc"),
+      },
+      gstCapsFilter{
+          QGstElement::createFromFactory("capsfilter", "videoCapsFilter"),
+      },
+      gstDecode{
+          QGstElement::createFromFactory("identity"),
+      },
+      gstVideoConvert{
+          QGstElement::createFromFactory("videoconvert", "videoConvert"),
+      },
+      gstVideoScale{
+          QGstElement::createFromFactory("videoscale", "videoScale"),
+      }
 {
-    gstDecode = QGstElement::createFromFactory("identity");
-    gstCameraBin = QGstBin::create("camerabin");
     gstCameraBin.add(gstCamera, gstCapsFilter, gstDecode, gstVideoConvert, gstVideoScale);
     qLinkGstElements(gstCamera, gstCapsFilter, gstDecode, gstVideoConvert, gstVideoScale);
     gstCameraBin.addGhostPad(gstVideoScale, "src");
@@ -79,6 +80,8 @@ void QGstreamerCamera::setActive(bool active)
 
 void QGstreamerCamera::setCamera(const QCameraDevice &camera)
 {
+    using namespace Qt::Literals;
+
     if (m_cameraDevice == camera)
         return;
 
@@ -89,12 +92,24 @@ void QGstreamerCamera::setCamera(const QCameraDevice &camera)
         gstNewCamera = QGstElement::createFromFactory("videotestsrc");
     } else {
         auto *integration = static_cast<QGstreamerIntegration *>(QGstreamerIntegration::instance());
-        auto *device = integration->videoDevice(camera.id());
-        gstNewCamera = QGstElement(gst_device_create_element(device, "camerasrc"));
-        if (QGstStructure properties = gst_device_get_properties(device); !properties.isNull()) {
-            if (properties.name() == "v4l2deviceprovider")
-                m_v4l2DevicePath = QString::fromUtf8(properties["device.path"].toString());
-            properties.free();
+        GstDevice *device = integration->videoDevice(camera.id());
+
+        if (!device) {
+            updateError(QCamera::Error::CameraError,
+                        u"Failed to create GstDevice for camera: "_s
+                                + QString::fromUtf8(camera.id()));
+            return;
+        }
+
+        gstNewCamera = QGstElement::createFromDevice(device, "camerasrc");
+        QUniqueGstStructureHandle properties{
+            gst_device_get_properties(device),
+        };
+
+        if (properties) {
+            QGstStructureView propertiesView{ properties };
+            if (propertiesView.name() == "v4l2deviceprovider")
+                m_v4l2DevicePath = QString::fromUtf8(propertiesView["device.path"].toString());
         }
     }
 
@@ -103,23 +118,22 @@ void QGstreamerCamera::setCamera(const QCameraDevice &camera)
     auto gstNewDecode = QGstElement::createFromFactory(
             f.pixelFormat() == QVideoFrameFormat::Format_Jpeg ? "jpegdec" : "identity");
 
-    qUnlinkGstElements(gstCamera, gstCapsFilter, gstDecode, gstVideoConvert);
-    gstCameraBin.stopAndRemoveElements(gstCamera, gstDecode);
+    m_currentCameraFormat = f;
 
-    gstCapsFilter.set("caps", caps);
+    updateCamera([&] {
+        qUnlinkGstElements(gstCamera, gstCapsFilter, gstDecode, gstVideoConvert);
+        gstCameraBin.stopAndRemoveElements(gstCamera, gstDecode);
 
-    gstCameraBin.add(gstNewCamera, gstNewDecode);
-    qLinkGstElements(gstNewCamera, gstCapsFilter, gstNewDecode, gstVideoConvert);
+        gstCapsFilter.set("caps", caps);
 
-    // Start sending frames once pipeline is linked
-    // FIXME: put camera to READY state before linking to decoder as in the NULL state it does not
-    // know its true caps
+        gstCamera = std::move(gstNewCamera);
+        gstDecode = std::move(gstNewDecode);
 
-    gstCameraBin.syncChildrenState();
+        gstCameraBin.add(gstCamera, gstDecode);
+        qLinkGstElements(gstCamera, gstCapsFilter, gstDecode, gstVideoConvert);
 
-
-    gstCamera = gstNewCamera;
-    gstDecode = gstNewDecode;
+        gstCameraBin.syncChildrenState();
+    });
 
     updateCameraProperties();
 }
@@ -133,24 +147,29 @@ bool QGstreamerCamera::setCameraFormat(const QCameraFormat &format)
     if (f.isNull())
         f = findBestCameraFormat(m_cameraDevice);
 
+    if (f == m_currentCameraFormat)
+        return true;
+
+    m_currentCameraFormat = f;
+
     auto caps = QGstCaps::fromCameraFormat(f);
 
     auto newGstDecode = QGstElement::createFromFactory(
             f.pixelFormat() == QVideoFrameFormat::Format_Jpeg ? "jpegdec" : "identity");
-    gstCameraBin.add(newGstDecode);
-    newGstDecode.syncStateWithParent();
 
-    gstCamera.staticPad("src").doInIdleProbe([&]() {
+    updateCamera([&] {
         qUnlinkGstElements(gstCamera, gstCapsFilter, gstDecode, gstVideoConvert);
+        gstCameraBin.stopAndRemoveElements(gstDecode);
 
         gstCapsFilter.set("caps", caps);
-        qLinkGstElements(gstCamera, gstCapsFilter, newGstDecode, gstVideoConvert);
+
+        gstDecode = std::move(newGstDecode);
+
+        gstCameraBin.add(gstDecode);
+        qLinkGstElements(gstCamera, gstCapsFilter, gstDecode, gstVideoConvert);
+        gstCameraBin.syncChildrenState();
     });
 
-    gstCameraBin.stopAndRemoveElements(gstDecode);
-
-    gstDecode = newGstDecode;
-
     return true;
 }
 
@@ -175,7 +194,7 @@ void QGstreamerCamera::updateCameraProperties()
 #if QT_CONFIG(gstreamer_photography)
 GstPhotography *QGstreamerCamera::photography() const
 {
-    if (!gstCamera.isNull() && GST_IS_PHOTOGRAPHY(gstCamera.element()))
+    if (gstCamera && GST_IS_PHOTOGRAPHY(gstCamera.element()))
         return GST_PHOTOGRAPHY(gstCamera.element());
     return nullptr;
 }
@@ -494,11 +513,9 @@ bool QGstreamerCamera::isWhiteBalanceModeSupported(QCamera::WhiteBalanceMode mod
         case QCamera::WhiteBalanceFluorescent:
             return true;
         case QCamera::WhiteBalanceManual: {
-#if GST_CHECK_VERSION(1, 18, 0)
             GstPhotographyInterface *iface = GST_PHOTOGRAPHY_GET_INTERFACE(p);
             if (iface->set_color_temperature && iface->get_color_temperature)
                 return true;
-#endif
             break;
         }
         default:
@@ -577,7 +594,7 @@ void QGstreamerCamera::setColorTemperature(int temperature)
     }
 #endif
 
-#if QT_CONFIG(gstreamer_photography) && GST_CHECK_VERSION(1, 18, 0)
+#if QT_CONFIG(gstreamer_photography)
     if (auto *p = photography()) {
         GstPhotographyInterface *iface = GST_PHOTOGRAPHY_GET_INTERFACE(p);
         Q_ASSERT(iface->set_color_temperature);
@@ -703,8 +720,53 @@ int QGstreamerCamera::getV4L2Parameter(quint32 id) const
     });
 }
 
-#endif
+#endif // QT_CONFIG(linux_v4l)
+
+QGstreamerCustomCamera::QGstreamerCustomCamera(QCamera *camera)
+    : QGstreamerCameraBase{
+          camera,
+      },
+      m_userProvidedGstElement{
+          false,
+      }
+{
+}
+
+QGstreamerCustomCamera::QGstreamerCustomCamera(QCamera *camera, QGstElement element)
+    : QGstreamerCameraBase{
+          camera,
+      },
+      gstCamera{
+          std::move(element),
+      },
+      m_userProvidedGstElement{
+          true,
+      }
+{
+}
+
+void QGstreamerCustomCamera::setCamera(const QCameraDevice &device)
+{
+    if (m_userProvidedGstElement)
+        return;
+
+    gstCamera = QGstBin::createFromPipelineDescription(device.id(), /*name=*/nullptr,
+                                                       /* ghostUnlinkedPads=*/true);
+}
+
+bool QGstreamerCustomCamera::isActive() const
+{
+    return m_active;
+}
+
+void QGstreamerCustomCamera::setActive(bool active)
+{
+    if (m_active == active)
+        return;
+
+    m_active = active;
+
+    emit activeChanged(active);
+}
 
 QT_END_NAMESPACE
-
-#include "moc_qgstreamercamera_p.cpp"
