@@ -103,7 +103,7 @@ QGStreamerPlatformSpecificInterfaceImplementation::gstPipeline(QMediaCaptureSess
     return gstreamerCapture ? gstreamerCapture->pipeline().pipeline() : nullptr;
 }
 
-Q_LOGGING_CATEGORY(lcGstreamer, "qt.multimedia.gstreamer")
+Q_STATIC_LOGGING_CATEGORY(lcGstreamer, "qt.multimedia.gstreamer")
 
 namespace {
 
@@ -145,12 +145,13 @@ QGstreamerIntegration::QGstreamerIntegration()
     : QPlatformMediaIntegration(QLatin1String("gstreamer"))
 {
     gst_init(nullptr, nullptr);
-    qCDebug(lcGstreamer) << "Using gstreamer version: " << gst_version_string();
+
+    const QGString version{ gst_version_string() };
+    qCInfo(lcGstreamer) << "Using Qt multimedia with GStreamer version:" << version.asStringView();
 
     GstRegistry *reg = gst_registry_get();
 
     if constexpr (!GST_CHECK_VERSION(1, 22, 0)) {
-        GstRegistry* reg = gst_registry_get();
         for (const char *name : vaapiPluginNames)
             rankDownPlugin(reg, name);
     }
@@ -193,22 +194,22 @@ const QGstreamerFormatInfo *QGstreamerIntegration::gstFormatsInfo()
     return static_cast<const QGstreamerFormatInfo *>(formatInfo());
 }
 
-QMaybe<QPlatformAudioDecoder *> QGstreamerIntegration::createAudioDecoder(QAudioDecoder *decoder)
+q23::expected<QPlatformAudioDecoder *, QString> QGstreamerIntegration::createAudioDecoder(QAudioDecoder *decoder)
 {
     return QGstreamerAudioDecoder::create(decoder);
 }
 
-QMaybe<QPlatformMediaCaptureSession *> QGstreamerIntegration::createCaptureSession()
+q23::expected<QPlatformMediaCaptureSession *, QString> QGstreamerIntegration::createCaptureSession()
 {
     return QGstreamerMediaCaptureSession::create();
 }
 
-QMaybe<QPlatformMediaPlayer *> QGstreamerIntegration::createPlayer(QMediaPlayer *player)
+q23::expected<QPlatformMediaPlayer *, QString> QGstreamerIntegration::createPlayer(QMediaPlayer *player)
 {
     return QGstreamerMediaPlayer::create(player);
 }
 
-QMaybe<QPlatformCamera *> QGstreamerIntegration::createCamera(QCamera *camera)
+q23::expected<QPlatformCamera *, QString> QGstreamerIntegration::createCamera(QCamera *camera)
 {
     if (inCustomCameraConstruction) {
         QGstElement element = std::exchange(pendingCameraElement, {});
@@ -219,27 +220,27 @@ QMaybe<QPlatformCamera *> QGstreamerIntegration::createCamera(QCamera *camera)
     return QGstreamerCamera::create(camera);
 }
 
-QMaybe<QPlatformMediaRecorder *> QGstreamerIntegration::createRecorder(QMediaRecorder *recorder)
+q23::expected<QPlatformMediaRecorder *, QString> QGstreamerIntegration::createRecorder(QMediaRecorder *recorder)
 {
     return new QGstreamerMediaRecorder(recorder);
 }
 
-QMaybe<QPlatformImageCapture *> QGstreamerIntegration::createImageCapture(QImageCapture *imageCapture)
+q23::expected<QPlatformImageCapture *, QString> QGstreamerIntegration::createImageCapture(QImageCapture *imageCapture)
 {
     return QGstreamerImageCapture::create(imageCapture);
 }
 
-QMaybe<QPlatformVideoSink *> QGstreamerIntegration::createVideoSink(QVideoSink *sink)
+q23::expected<QPlatformVideoSink *, QString> QGstreamerIntegration::createVideoSink(QVideoSink *sink)
 {
     return new QGstreamerVideoSink(sink);
 }
 
-QMaybe<QPlatformAudioInput *> QGstreamerIntegration::createAudioInput(QAudioInput *q)
+q23::expected<QPlatformAudioInput *, QString> QGstreamerIntegration::createAudioInput(QAudioInput *q)
 {
     return QGstreamerAudioInput::create(q);
 }
 
-QMaybe<QPlatformAudioOutput *> QGstreamerIntegration::createAudioOutput(QAudioOutput *q)
+q23::expected<QPlatformAudioOutput *, QString> QGstreamerIntegration::createAudioOutput(QAudioOutput *q)
 {
     return QGstreamerAudioOutput::create(q);
 }
