@@ -55,6 +55,11 @@ private:
     bool m_isContinuous;
 };
 
+static bool isPipewireBackend()
+{
+    return QPlatformMediaIntegration::audioBackendName() == "PipeWire";
+}
+
 static bool isPulseAudioBackend()
 {
     return QPlatformMediaIntegration::audioBackendName() == "PulseAudio";
@@ -62,11 +67,11 @@ static bool isPulseAudioBackend()
 
 static bool underrunIsAnError()
 {
-#ifdef Q_OS_APPLE
+#if defined(Q_OS_APPLE) || defined(Q_OS_ANDROID)
     return false;
 #endif
 
-    return !(isPulseAudioBackend());
+    return !(isPipewireBackend() || isPulseAudioBackend());
 }
 
 class tst_QAudioSink : public QObject
@@ -82,6 +87,7 @@ private slots:
     void format();
     void invalidFormat_data();
     void invalidFormat();
+    void nullFormat();
 
     void bufferSize_data();
     void bufferSize();
@@ -121,12 +127,15 @@ private slots:
 
     void callbackAPI();
     void callbackAPI_startFailsWithWrongType();
+    void callbackAPI_startWithMoveOnlyFunctor();
 
     void multipleSinks_data() { generate_multiple_sinks_testrows(); }
     void multipleSinks();
     void start_afterStopAndReset_data() { generate_multiple_sinks_testrows(); }
     void start_afterStopAndReset();
 
+    void destroy_while_running(); // should be last test to catch crash on exit
+
 private:
     using FilePtr = std::shared_ptr<QFile>;
 
@@ -241,9 +250,9 @@ void tst_QAudioSink::generate_multiple_sinks_testrows()
 
 void tst_QAudioSink::initTestCase()
 {
-    // Only perform tests if audio output device exists
     const QList<QAudioDevice> devices = QMediaDevices::audioOutputs();
 
+    // Only perform tests if audio output device exists
     if (devices.isEmpty())
         QSKIP("No audio outputs found");
 
@@ -285,7 +294,8 @@ void tst_QAudioSink::initTestCase()
         });
 #endif
 
-    QVERIFY(testFormats.size());
+    if (testFormats.empty())
+        QSKIP("audio devices does not support test format"); // e.g RME Fireface / pipewire
 
     const QChar slash = QLatin1Char('/');
     QString temporaryPattern = QDir::tempPath();
@@ -354,9 +364,6 @@ void tst_QAudioSink::invalidFormat_data()
 
     QAudioFormat format;
 
-    QTest::newRow("Null Format")
-            << format;
-
     format = audioDevice.preferredFormat();
     format.setChannelCount(0);
     QTest::newRow("Channel count 0")
@@ -396,6 +403,22 @@ void tst_QAudioSink::invalidFormat()
                  "error() was not set to QAudio::OpenError after start()");
 }
 
+void tst_QAudioSink::nullFormat()
+{
+    QAudioDevice audioDevice = QMediaDevices::defaultAudioOutput();
+    if (audioDevice.isNull())
+        QSKIP("No audio outputs found");
+
+    {
+        QAudioSink audioSink;
+        QCOMPARE(audioSink.format(), audioDevice.preferredFormat());
+    }
+    {
+        QAudioSink audioSink(audioDevice);
+        QCOMPARE(audioSink.format(), audioDevice.preferredFormat());
+    }
+}
+
 void tst_QAudioSink::bufferSize_data()
 {
     QTest::addColumn<int>("bufferSize");
@@ -548,7 +571,7 @@ void tst_QAudioSink::pull()
     QVERIFY2((audioSink.elapsedUSecs() == qint64(0)), "elapsedUSecs() not zero on creation");
 
     audioFile->close();
-    audioFile->open(QIODevice::ReadOnly);
+    QVERIFY(audioFile->open(QIODevice::ReadOnly));
     audioFile->seek(QWaveDecoder::headerLength());
 
     audioSink.start(audioFile.get());
@@ -614,7 +637,7 @@ void tst_QAudioSink::pullSuspendResume()
     QVERIFY2((audioSink.elapsedUSecs() == qint64(0)), "elapsedUSecs() not zero on creation");
 
     audioFile->close();
-    audioFile->open(QIODevice::ReadOnly);
+    QVERIFY(audioFile->open(QIODevice::ReadOnly));
     audioFile->seek(QWaveDecoder::headerLength());
 
     audioSink.start(audioFile.get());
@@ -729,7 +752,6 @@ void tst_QAudioSink::pullResumeFromUnderrun()
     QCOMPARE(audioSink.state(), QAudio::IdleState);
     if (underrunIsAnError())
         QCOMPARE(audioSink.error(), QAudio::UnderrunError);
-
     stateSignal.clear();
 
     QTest::qWait(300);
@@ -772,7 +794,7 @@ void tst_QAudioSink::push()
     QVERIFY2((audioSink.elapsedUSecs() == qint64(0)), "elapsedUSecs() not zero on creation");
 
     audioFile->close();
-    audioFile->open(QIODevice::ReadOnly);
+    QVERIFY(audioFile->open(QIODevice::ReadOnly));
     audioFile->seek(QWaveDecoder::headerLength());
 
     QIODevice *feed = audioSink.start();
@@ -840,7 +862,7 @@ void tst_QAudioSink::push()
     QVERIFY2((audioSink.elapsedUSecs() == (qint64)0),
              "elapsedUSecs() not equal to zero in StoppedState");
 
-    if (isWindows || isMacOS || isPulseAudioBackend()) {
+    if (isWindows || isMacOS || isPulseAudioBackend() || isPipewireBackend()) {
         QVERIFY(!dataWrittenToDeviceSpy.empty());
         auto allBytesWritten = [&] {
             qint64 total = 0;
@@ -873,7 +895,7 @@ void tst_QAudioSink::pushSuspendResume()
     QVERIFY2((audioSink.elapsedUSecs() == qint64(0)), "elapsedUSecs() not zero on creation");
 
     audioFile->close();
-    audioFile->open(QIODevice::ReadOnly);
+    QVERIFY(audioFile->open(QIODevice::ReadOnly));
     audioFile->seek(QWaveDecoder::headerLength());
 
     QIODevice *feed = audioSink.start();
@@ -1009,7 +1031,7 @@ void tst_QAudioSink::pushResetResume()
     audioSink.setVolume(0.1f);
 
     audioFile->close();
-    audioFile->open(QIODevice::ReadOnly);
+    QVERIFY(audioFile->open(QIODevice::ReadOnly));
     audioFile->seek(QWaveDecoder::headerLength());
 
     QPointer<QIODevice> feed = audioSink.start();
@@ -1057,7 +1079,7 @@ void tst_QAudioSink::pushUnderrun()
     QVERIFY2((audioSink.elapsedUSecs() == qint64(0)), "elapsedUSecs() not zero on creation");
 
     audioFile->close();
-    audioFile->open(QIODevice::ReadOnly);
+    QVERIFY(audioFile->open(QIODevice::ReadOnly));
     audioFile->seek(QWaveDecoder::headerLength());
 
     QIODevice *feed = audioSink.start();
@@ -1112,7 +1134,6 @@ void tst_QAudioSink::pushUnderrun()
     if (underrunIsAnError())
         QVERIFY2((audioSink.error() == QAudio::UnderrunError),
                  "error state is not equal to QAudio::UnderrunError, no data");
-
     stateSignal.clear();
 
     // Play rest of the clip
@@ -1226,7 +1247,7 @@ void tst_QAudioSink::stateChanged_stringBasedConnect()
 {
     const QAudioDevice defaultAudioOutputDevice = QMediaDevices::defaultAudioOutput();
 
-    QAudioSink audiosink(defaultAudioOutputDevice, defaultAudioOutputDevice.preferredFormat());
+    QAudioSink audiosink(defaultAudioOutputDevice);
 
     QSignalSpy stateSignal(&audiosink, SIGNAL(stateChanged(QAudio::State)));
 
@@ -1279,6 +1300,34 @@ void tst_QAudioSink::callbackAPI_startFailsWithWrongType()
     QCOMPARE(audioSink.error(), QAudio::Error::OpenError);
 }
 
+void tst_QAudioSink::callbackAPI_startWithMoveOnlyFunctor()
+{
+#if QT_CONFIG(thread) && defined(__cpp_lib_move_only_function)
+    using namespace std::chrono_literals;
+
+    QAudioFormat format = audioDevice.preferredFormat();
+    format.setSampleFormat(QAudioFormat::SampleFormat::Float);
+
+    QAudioSink audioSink(audioDevice, format);
+    QPlatformAudioSink *platformSink = QPlatformAudioSink::get(audioSink);
+    if (!platformSink->hasCallbackAPI())
+        QSKIP("Callback API not supported by this backend");
+
+    QSemaphore sync;
+
+    platformSink->start([&, dummy = std::make_unique<int>(1)](QSpan<float> outputBuffer) {
+        QCOMPARE_GT(outputBuffer.size(), 0);
+        sync.release();
+    });
+    QCOMPARE(audioSink.error(), QAudio::Error::NoError);
+
+    bool callbackExecuted = sync.try_acquire_for(1s);
+    QVERIFY(callbackExecuted);
+#else
+    QSKIP("Threading not configured or move-only functions not available");
+#endif
+}
+
 void tst_QAudioSink::multipleSinks()
 {
     QFETCH(QAudioDevice, firstSinkDevice);
@@ -1339,6 +1388,17 @@ void tst_QAudioSink::start_afterStopAndReset()
     QTRY_COMPARE_GT(sink->processedUSecs(), 0);
 }
 
+void tst_QAudioSink::destroy_while_running()
+{
+    AudioPullSource src;
+    src.open(QIODeviceBase::ReadOnly);
+    src.available = 44100 * 10; // 10 seconds of audio
+
+    QAudioSink sink(audioDevice, audioDevice.preferredFormat());
+
+    sink.start(&src);
+}
+
 QTEST_MAIN(tst_QAudioSink)
 
 #include "tst_qaudiosink.moc"
