@@ -50,6 +50,7 @@ private slots:
     void format();
     void invalidFormat_data();
     void invalidFormat();
+    void nullFormat();
 
     void bufferSize();
     void bufferSize_getValidDefault();
@@ -131,15 +132,6 @@ QString tst_QAudioSource::formatToFileName(const QAudioFormat &format)
 
 void tst_QAudioSource::initTestCase()
 {
-#ifdef Q_OS_ANDROID
-    // The test might fail because libOpenSLES cannot create AudioRecorder for that emulator. The
-    // Android documentation states that the emulator doesn't support this at all all
-    // https://developer.android.com/media/platform/mediarecorder. However, in practice this test
-    // fails only prior to Android 10.
-    if (QNativeInterface::QAndroidApplication::sdkVersion() < __ANDROID_API_Q__)
-        QSKIP("Emulated Android version doesn't support audio recording");
-#endif
-
     if (m_inCISystem)
         QSKIP("SKIP initTestCase on CI. To be fixed");
 
@@ -223,9 +215,6 @@ void tst_QAudioSource::invalidFormat_data()
 
     QAudioFormat format;
 
-    QTest::newRow("Null Format")
-            << format;
-
     format = audioDevice.preferredFormat();
     format.setChannelCount(0);
     QTest::newRow("Channel count 0")
@@ -265,6 +254,22 @@ void tst_QAudioSource::invalidFormat()
                  "error() was not set to QAudio::OpenError after start()");
 }
 
+void tst_QAudioSource::nullFormat()
+{
+    QAudioDevice audioDevice = QMediaDevices::defaultAudioInput();
+    if (audioDevice.isNull())
+        QSKIP("No audio inputs found");
+
+    {
+        QAudioSource audioSource;
+        QCOMPARE(audioSource.format(), audioDevice.preferredFormat());
+    }
+    {
+        QAudioSource audioSource(audioDevice);
+        QCOMPARE(audioSource.format(), audioDevice.preferredFormat());
+    }
+}
+
 void tst_QAudioSource::bufferSize()
 {
     QAudioSource audioSource(audioDevice.preferredFormat(), this);
@@ -1016,7 +1021,7 @@ void tst_QAudioSource::stateChanged_stringBasedConnect()
 {
     const QAudioDevice defaultAudioInputDevice = QMediaDevices::defaultAudioInput();
 
-    QAudioSource audioSource(defaultAudioInputDevice, defaultAudioInputDevice.preferredFormat());
+    QAudioSource audioSource(defaultAudioInputDevice);
 
     QSignalSpy stateSignal(&audioSource, SIGNAL(stateChanged(QAudio::State)));
 
