@@ -130,7 +130,7 @@ void tst_QWaveDecoder::file()
 
     QFile stream;
     stream.setFileName(file);
-    stream.open(QIODevice::ReadOnly);
+    QVERIFY(stream.open(QIODevice::ReadOnly));
 
     QVERIFY(stream.isOpen());
 
@@ -212,7 +212,7 @@ void tst_QWaveDecoder::http()
 
     QFile stream;
     stream.setFileName(file);
-    stream.open(QIODevice::ReadOnly);
+    QVERIFY(stream.open(QIODevice::ReadOnly));
 
     QVERIFY(stream.isOpen());
 
@@ -267,7 +267,7 @@ void tst_QWaveDecoder::readAllAtOnce()
 {
     QFile stream;
     stream.setFileName(testFilePath("isawav_2_8_44100.wav"));
-    stream.open(QIODevice::ReadOnly);
+    QVERIFY(stream.open(QIODevice::ReadOnly));
 
     QVERIFY(stream.isOpen());
 
@@ -295,7 +295,7 @@ void tst_QWaveDecoder::readPerByte()
 {
     QFile stream;
     stream.setFileName(testFilePath("isawav_2_8_44100.wav"));
-    stream.open(QIODevice::ReadOnly);
+    QVERIFY(stream.open(QIODevice::ReadOnly));
 
     QVERIFY(stream.isOpen());
 
