@@ -1,5 +1,5 @@
 // Copyright (C) 2023 The Qt Company Ltd.
-// SPDX-License-Identifier: LicenseRef-Qt-Commercial OR LGPL-3.0-only OR GPL-2.0-only OR GPL-3.0-only
+// SPDX-License-Identifier: LicenseRef-Qt-Commercial OR GPL-3.0-only
 
 #ifndef MAINWINDOW_H
 #define MAINWINDOW_H
@@ -12,6 +12,8 @@
 #include <QMediaDevices>
 #include <QMediaRecorder>
 
+#include <memory>
+
 QT_BEGIN_NAMESPACE
 namespace Ui { class MainWindow; }
 
@@ -41,9 +43,9 @@ private:
     void enableButtons(bool ok);
     void showFile(const QString &filename);
 
-    QScopedPointer<QCamera> m_camera;
+    std::unique_ptr<QCamera> m_camera;
     QMediaCaptureSession *m_captureSession;
-    QScopedPointer<QAudioInput> m_audioInput;
+    std::unique_ptr<QAudioInput> m_audioInput;
     QMediaDevices *m_mediaDevices;
     QMediaRecorder *m_recorder;
     bool isRecording = false;
